@@ -44,7 +44,7 @@
 #endif /* GRN_WITH_ONIGMO */
 
 #ifdef GRN_SUPPORT_REGEXP
-# include <oniguruma.h>
+# include <onigmo.h>
 #endif /* GRN_SUPPORT_REGEXP */
 
 #ifdef WIN32
@@ -147,6 +147,7 @@ grn_loader_init(grn_loader *loader)
   GRN_TEXT_INIT(&loader->values, 0);
   GRN_UINT32_INIT(&loader->level, GRN_OBJ_VECTOR);
   GRN_PTR_INIT(&loader->columns, GRN_OBJ_VECTOR, GRN_ID_NIL);
+  GRN_UINT32_INIT(&loader->ids, GRN_OBJ_VECTOR);
   loader->id_offset = -1;
   loader->key_offset = -1;
   loader->table = NULL;
@@ -159,6 +160,7 @@ grn_loader_init(grn_loader *loader)
   loader->columns_status = GRN_LOADER_COLUMNS_UNSET;
   loader->rc = GRN_SUCCESS;
   loader->errbuf[0] = '\0';
+  loader->output_ids = GRN_FALSE;
 }
 
 void
@@ -176,6 +178,7 @@ grn_ctx_loader_clear(grn_ctx *ctx)
   GRN_OBJ_FIN(ctx, &loader->values);
   GRN_OBJ_FIN(ctx, &loader->level);
   GRN_OBJ_FIN(ctx, &loader->columns);
+  GRN_OBJ_FIN(ctx, &loader->ids);
   grn_loader_init(loader);
 }
 
@@ -234,7 +237,6 @@ grn_ctx_impl_init(grn_ctx *ctx)
   ctx->impl->expr_vars = grn_hash_create(ctx, NULL, sizeof(grn_id), sizeof(grn_obj *), 0);
   ctx->impl->stack_curr = 0;
   ctx->impl->curr_expr = NULL;
-  ctx->impl->qe_next = NULL;
   GRN_TEXT_INIT(&ctx->impl->current_request_id, 0);
   ctx->impl->current_request_timer_id = NULL;
   ctx->impl->parser = NULL;
@@ -242,11 +244,14 @@ grn_ctx_impl_init(grn_ctx *ctx)
   GRN_TEXT_INIT(&ctx->impl->output.names, GRN_OBJ_VECTOR);
   GRN_UINT32_INIT(&ctx->impl->output.levels, GRN_OBJ_VECTOR);
 
+  ctx->impl->command.flags = 0;
   if (ctx == &grn_gctx) {
-    ctx->impl->command_version = GRN_COMMAND_VERSION_STABLE;
+    ctx->impl->command.version = GRN_COMMAND_VERSION_STABLE;
   } else {
-    ctx->impl->command_version = grn_get_default_command_version();
+    ctx->impl->command.version = grn_get_default_command_version();
   }
+  ctx->impl->command.keep.command = NULL;
+  ctx->impl->command.keep.version = ctx->impl->command.version;
 
   if (ctx == &grn_gctx) {
     ctx->impl->match_escalation_threshold =
@@ -279,13 +284,17 @@ grn_ctx_impl_init(grn_ctx *ctx)
 
   grn_ctx_impl_mrb_init(ctx);
 
+  GRN_TEXT_INIT(&(ctx->impl->temporary_open_spaces.stack), 0);
+  ctx->impl->temporary_open_spaces.current = NULL;
+
   return ctx->rc;
 }
 
 void
-grn_ctx_set_next_expr(grn_ctx *ctx, grn_obj *expr)
+grn_ctx_set_keep_command(grn_ctx *ctx, grn_obj *command)
 {
-  ctx->impl->qe_next = expr;
+  ctx->impl->command.keep.command = command;
+  ctx->impl->command.keep.version = ctx->impl->command.version;
 }
 
 static void
@@ -411,6 +420,20 @@ grn_ctx_fin(grn_ctx *ctx)
     if (ctx->impl->finalizer) {
       ctx->impl->finalizer(ctx, 0, NULL, &(ctx->user_data));
     }
+    {
+      grn_obj *stack;
+      grn_obj *spaces;
+      unsigned int i, n_spaces;
+
+      stack = &(ctx->impl->temporary_open_spaces.stack);
+      spaces = (grn_obj *)GRN_BULK_HEAD(stack);
+      n_spaces = GRN_BULK_VSIZE(stack) / sizeof(grn_obj);
+      for (i = 0; i < n_spaces; i++) {
+        grn_obj *space = spaces + (n_spaces - i - 1);
+        GRN_OBJ_FIN(ctx, space);
+      }
+      GRN_OBJ_FIN(ctx, stack);
+    }
     grn_ctx_impl_mrb_fin(ctx);
     grn_ctx_loader_clear(ctx);
     if (ctx->impl->parser) {
@@ -755,7 +778,7 @@ grn_command_version
 grn_ctx_get_command_version(grn_ctx *ctx)
 {
   if (ctx->impl) {
-    return ctx->impl->command_version;
+    return ctx->impl->command.version;
   } else {
     return GRN_COMMAND_VERSION_STABLE;
   }
@@ -766,12 +789,12 @@ grn_ctx_set_command_version(grn_ctx *ctx, grn_command_version version)
 {
   switch (version) {
   case GRN_COMMAND_VERSION_DEFAULT :
-    ctx->impl->command_version = GRN_COMMAND_VERSION_STABLE;
+    ctx->impl->command.version = GRN_COMMAND_VERSION_STABLE;
     return GRN_SUCCESS;
   default :
     if (GRN_COMMAND_VERSION_MIN <= version &&
         version <= GRN_COMMAND_VERSION_MAX) {
-      ctx->impl->command_version = version;
+      ctx->impl->command.version = version;
       return GRN_SUCCESS;
     } else {
       return GRN_UNSUPPORTED_COMMAND_VERSION;
@@ -1273,12 +1296,14 @@ grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
   if (!ctx) { return 0; }
   GRN_API_ENTER;
   if (ctx->impl) {
+    if ((flags & GRN_CTX_MORE)) { flags |= GRN_CTX_QUIET; }
+    if (ctx->stat == GRN_CTX_QUIT) { flags |= GRN_CTX_QUIT; }
+
+    ctx->impl->command.flags = flags;
     if (ctx->impl->com) {
       grn_rc rc;
       grn_com_header sheader;
       grn_timeval_now(ctx, &ctx->impl->tv);
-      if ((flags & GRN_CTX_MORE)) { flags |= GRN_CTX_QUIET; }
-      if (ctx->stat == GRN_CTX_QUIT) { flags |= GRN_CTX_QUIT; }
       sheader.proto = GRN_COM_PROTO_GQTP;
       sheader.qtype = 0;
       sheader.keylen = 0;
@@ -1296,10 +1321,11 @@ grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
       grn_obj *expr = NULL;
 
       command_version = grn_ctx_get_command_version(ctx);
-      if (ctx->impl->qe_next) {
+      if (ctx->impl->command.keep.command) {
         grn_obj *val;
-        expr = ctx->impl->qe_next;
-        ctx->impl->qe_next = NULL;
+        expr = ctx->impl->command.keep.command;
+        ctx->impl->command.keep.command = NULL;
+        grn_ctx_set_command_version(ctx, ctx->impl->command.keep.version);
         if ((val = grn_expr_get_var_by_offset(ctx, expr, 0))) {
           grn_obj_reinit(ctx, val, GRN_DB_TEXT, 0);
           GRN_TEXT_PUT(ctx, val, str, str_len);
@@ -1321,7 +1347,7 @@ grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
         }
       }
       if (ctx->stat == GRN_CTX_QUITTING) { ctx->stat = GRN_CTX_QUIT; }
-      if (ctx->impl->qe_next) {
+      if (ctx->impl->command.keep.command) {
         ERRCLR(ctx);
       } else {
         if (ctx->impl->current_request_timer_id) {
@@ -1340,7 +1366,8 @@ grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
                       "<", "rc=%d", ctx->rc);
       }
     output :
-      if (!(flags & GRN_CTX_QUIET) && ctx->impl->output.func) {
+      if (!(ctx->impl->command.flags & GRN_CTX_QUIET) &&
+          ctx->impl->output.func) {
         ctx->impl->output.func(ctx, GRN_CTX_TAIL, ctx->impl->output.data.ptr);
       }
       if (expr) { grn_expr_clear_vars(ctx, expr); }
