@@ -198,7 +198,8 @@ grn_dat_open_trie_if_needed(grn_ctx *ctx, grn_dat *dat)
     new_trie->open(trie_path);
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::open failed");
+        "grn::dat::Trie::open failed: %s",
+        ex.what());
     delete new_trie;
     return false;
   }
@@ -232,7 +233,8 @@ bool grn_dat_rebuild_trie(grn_ctx *ctx, grn_dat *dat) {
     new_trie->create(*trie, trie_path, trie->file_size() * 2);
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::open failed");
+        "grn::dat::Trie::open failed: %s",
+        ex.what());
     delete new_trie;
     return false;
   }
@@ -432,7 +434,8 @@ grn_dat_get(grn_ctx *ctx, grn_dat *dat, const void *key,
     }
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::search failed");
+        "grn::dat::Trie::search failed: %s",
+        ex.what());
   }
   return GRN_ID_NIL;
 }
@@ -459,7 +462,8 @@ grn_dat_add(grn_ctx *ctx, grn_dat *dat, const void *key,
       new_trie->create(trie_path);
     } catch (const grn::dat::Exception &ex) {
       ERR(grn_dat_translate_error_code(ex.code()),
-          "grn::dat::Trie::create failed");
+          "grn::dat::Trie::create failed: %s",
+          ex.what());
       delete new_trie;
       return GRN_ID_NIL;
     }
@@ -488,7 +492,8 @@ grn_dat_add(grn_ctx *ctx, grn_dat *dat, const void *key,
     return new_trie->get_key(key_pos).id();
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::insert failed");
+        "grn::dat::Trie::insert failed: %s",
+        ex.what());
     return GRN_ID_NIL;
   }
 }
@@ -562,7 +567,8 @@ grn_dat_delete_by_id(grn_ctx *ctx, grn_dat *dat, grn_id id,
     }
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::remove failed");
+        "grn::dat::Trie::remove failed: %s",
+        ex.what());
     return ctx->rc;
   }
   return GRN_SUCCESS;
@@ -590,7 +596,8 @@ grn_dat_delete(grn_ctx *ctx, grn_dat *dat, const void *key, unsigned int key_siz
       }
     } catch (const grn::dat::Exception &ex) {
       ERR(grn_dat_translate_error_code(ex.code()),
-          "grn::dat::Trie::search failed");
+          "grn::dat::Trie::search failed: %s",
+          ex.what());
       return ctx->rc;
     }
   }
@@ -602,7 +609,8 @@ grn_dat_delete(grn_ctx *ctx, grn_dat *dat, const void *key, unsigned int key_siz
     }
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::remove failed");
+        "grn::dat::Trie::remove failed: %s",
+        ex.what());
     return ctx->rc;
   }
   return GRN_SUCCESS;
@@ -636,7 +644,8 @@ grn_dat_update_by_id(grn_ctx *ctx, grn_dat *dat, grn_id src_key_id,
     }
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::update failed");
+        "grn::dat::Trie::update failed: %s",
+        ex.what());
     return ctx->rc;
   }
   return GRN_SUCCESS;
@@ -671,7 +680,8 @@ grn_dat_update(grn_ctx *ctx, grn_dat *dat,
     }
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::update failed");
+        "grn::dat::Trie::update failed: %s",
+        ex.what());
     return ctx->rc;
   }
   return GRN_SUCCESS;
@@ -791,7 +801,8 @@ grn_dat_scan(grn_ctx *ctx, grn_dat *dat, const char *str,
     }
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::lcp_search failed");
+        "grn::dat::lcp_search failed: %s",
+        ex.what());
     if (str_rest) {
       *str_rest = str;
     }
@@ -822,7 +833,8 @@ grn_dat_lcp_search(grn_ctx *ctx, grn_dat *dat,
     return trie->get_key(key_pos).id();
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::PrefixCursor::open failed");
+        "grn::dat::PrefixCursor::open failed: %s",
+        ex.what());
     return GRN_ID_NIL;
   }
 }
@@ -905,7 +917,8 @@ grn_dat_cursor_open(grn_ctx *ctx, grn_dat *dat,
     }
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::CursorFactory::open failed");
+        "grn::dat::CursorFactory::open failed: %s",
+        ex.what());
     GRN_FREE(dc);
     return NULL;
   }
@@ -931,7 +944,8 @@ grn_dat_cursor_next(grn_ctx *ctx, grn_dat_cursor *c)
     c->curr_rec = key.is_valid() ? key.id() : GRN_ID_NIL;
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Cursor::next failed");
+        "grn::dat::Cursor::next failed: %s",
+        ex.what());
     return GRN_ID_NIL;
   }
   return c->curr_rec;
@@ -978,7 +992,8 @@ grn_dat_cursor_delete(grn_ctx *ctx, grn_dat_cursor *c,
     }
   } catch (const grn::dat::Exception &ex) {
     ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::remove failed");
+        "grn::dat::Trie::remove failed: %s",
+        ex.what());
     return GRN_INVALID_ARGUMENT;
   }
   return GRN_INVALID_ARGUMENT;
@@ -1014,7 +1029,7 @@ grn_dat_truncate(grn_ctx *ctx, grn_dat *dat)
     grn::dat::Trie().create(trie_path);
   } catch (const grn::dat::Exception &ex) {
     const grn_rc error_code = grn_dat_translate_error_code(ex.code());
-    ERR(error_code, "grn::dat::Trie::create failed");
+    ERR(error_code, "grn::dat::Trie::create failed: %s", ex.what());
     return error_code;
   }
   ++dat->header->file_id;
@@ -1028,14 +1043,17 @@ const char *
 _grn_dat_key(grn_ctx *ctx, grn_dat *dat, grn_id id, uint32_t *key_size)
 {
   if (!grn_dat_open_trie_if_needed(ctx, dat)) {
+    *key_size = 0;
     return NULL;
   }
   const grn::dat::Trie * const trie = static_cast<grn::dat::Trie *>(dat->trie);
   if (!trie) {
+    *key_size = 0;
     return NULL;
   }
   const grn::dat::Key &key = trie->ith_key(id);
   if (!key.is_valid()) {
+    *key_size = 0;
     return NULL;
   }
   *key_size = key.length();
@@ -1108,7 +1126,7 @@ grn_dat_repair(grn_ctx *ctx, grn_dat *dat)
     grn::dat::Trie().repair(*trie, trie_path);
   } catch (const grn::dat::Exception &ex) {
     const grn_rc error_code = grn_dat_translate_error_code(ex.code());
-    ERR(error_code, "grn::dat::Trie::create failed");
+    ERR(error_code, "grn::dat::Trie::create failed: %s", ex.what());
     return error_code;
   }
   ++dat->header->file_id;
@@ -1137,9 +1155,9 @@ grn_dat_flush(grn_ctx *ctx, grn_dat *dat)
     } catch (const grn::dat::Exception &ex) {
       const grn_rc error_code = grn_dat_translate_error_code(ex.code());
       if (error_code == GRN_INPUT_OUTPUT_ERROR) {
-        SERR("grn::dat::Trie::flush failed");
+        SERR("grn::dat::Trie::flush failed: %s", ex.what());
       } else {
-        ERR(error_code, "grn::dat::Trie::flush failed");
+        ERR(error_code, "grn::dat::Trie::flush failed: %s", ex.what());
       }
       return error_code;
     }
@@ -1183,11 +1201,12 @@ grn_dat_is_dirty(grn_ctx *ctx, grn_dat *dat)
 grn_rc
 grn_dat_clean(grn_ctx *ctx, grn_dat *dat)
 {
-  if (!dat->io) {
-  }
-
   grn_rc rc = GRN_SUCCESS;
 
+  if (!dat->io) {
+    return rc;
+  }
+
   {
     CriticalSection critical_section(&dat->lock);
     if (dat->is_dirty) {
@@ -1201,4 +1220,23 @@ grn_dat_clean(grn_ctx *ctx, grn_dat *dat)
   return rc;
 }
 
+grn_rc
+grn_dat_clear_dirty(grn_ctx *ctx, grn_dat *dat)
+{
+  grn_rc rc = GRN_SUCCESS;
+
+  if (!dat->io) {
+    return rc;
+  }
+
+  {
+    CriticalSection critical_section(&dat->lock);
+    dat->is_dirty = GRN_FALSE;
+    dat->header->n_dirty_opens = 0;
+    rc = grn_io_flush(ctx, dat->io);
+  }
+
+  return rc;
+}
+
 }  // extern "C"
