@@ -109,13 +109,7 @@
 #  endif
 # endif
 
-# if !defined(__GNUC__) && _MSC_VER < 1500
-#  define vsnprintf(str, size, format, ap) _vsnprintf(str, size, format, ap)
-# endif /* !defined(__GNUC__) && _MSC_VER < 1500 */
 # define getpid() _getpid()
-# if !defined(__GNUC__) && _MSC_VER < 1400
-#  define fstat(fd, buf) _fstat(fd, buf)
-# endif /* !defined(__GNUC__) && _MSC_VER < 1400 */
 
 # ifndef __GNUC__
 typedef SSIZE_T ssize_t;
@@ -549,7 +543,7 @@ typedef int grn_cond;
 #elif (defined(WIN32) || defined (_WIN64)) /* __GNUC__ */
 
 # define GRN_ATOMIC_ADD_EX(p,i,r) \
-  ((r) = (uint32_t)InterlockedExchangeAdd((int32_t *)(p), (int32_t)(i)))
+  ((r) = InterlockedExchangeAdd((p), (i)))
 # if defined(_WIN64) /* ATOMIC 64BIT SET */
 #  define GRN_SET_64BIT(p,v) \
   (*(p) = (v))
