@@ -198,85 +198,4 @@ grn_mrb_load(grn_ctx *ctx, const char *path)
 
   return result;
 }
-
-mrb_value
-grn_mrb_eval(grn_ctx *ctx, const char *script, int script_length)
-{
-  grn_mrb_data *data = &(ctx->impl->mrb);
-  mrb_state *mrb = data->state;
-  mrb_value result;
-  struct mrb_parser_state *parser;
-
-  if (!mrb) {
-    return mrb_nil_value();
-  }
-
-  if (script_length < 0) {
-    script_length = strlen(script);
-  }
-  parser = mrb_parse_nstring(mrb, script, script_length, NULL);
-  {
-    struct RProc *proc;
-    struct RClass *eval_context_class;
-    mrb_value eval_context;
-
-    proc = mrb_generate_code(mrb, parser);
-    eval_context_class = mrb_class_get_under(mrb, data->module, "EvalContext");
-    eval_context = mrb_obj_new(mrb, eval_context_class, 0, NULL);
-    result = mrb_context_run(mrb, proc, eval_context, 0);
-  }
-  mrb_parser_free(parser);
-
-  return result;
-}
-
-grn_rc
-grn_mrb_to_grn(grn_ctx *ctx, mrb_value mrb_object, grn_obj *grn_object)
-{
-  grn_rc rc = GRN_SUCCESS;
-  grn_mrb_data *data = &(ctx->impl->mrb);
-  mrb_state *mrb = data->state;
-
-  switch (mrb_type(mrb_object)) {
-  case MRB_TT_FALSE :
-    if (mrb_nil_p(mrb_object)) {
-      grn_obj_reinit(ctx, grn_object, GRN_DB_VOID, 0);
-    } else {
-      grn_obj_reinit(ctx, grn_object, GRN_DB_BOOL, 0);
-      GRN_BOOL_SET(ctx, grn_object, GRN_FALSE);
-    }
-    break;
-  case MRB_TT_TRUE :
-    grn_obj_reinit(ctx, grn_object, GRN_DB_BOOL, 0);
-    GRN_BOOL_SET(ctx, grn_object, GRN_TRUE);
-    break;
-  case MRB_TT_FIXNUM :
-    grn_obj_reinit(ctx, grn_object, GRN_DB_INT32, 0);
-    GRN_INT32_SET(ctx, grn_object, mrb_fixnum(mrb_object));
-    break;
-  case MRB_TT_STRING :
-    grn_obj_reinit(ctx, grn_object, GRN_DB_TEXT, 0);
-    GRN_TEXT_SET(ctx, grn_object,
-                 RSTRING_PTR(mrb_object),
-                 RSTRING_LEN(mrb_object));
-    break;
-  case MRB_TT_SYMBOL :
-    {
-      const char *name;
-      int name_length;
-
-      grn_obj_reinit(ctx, grn_object, GRN_DB_TEXT, 0);
-      GRN_BULK_REWIND(grn_object);
-      GRN_TEXT_PUTC(ctx, grn_object, ':');
-      name = mrb_sym2name_len(mrb, mrb_symbol(mrb_object), &name_length);
-      GRN_TEXT_PUT(ctx, grn_object, name, name_length);
-    }
-    break;
-  default :
-    rc = GRN_INVALID_ARGUMENT;
-    break;
-  }
-
-  return rc;
-}
 #endif
