@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2014 Brazil
+  Copyright(C) 2014-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,10 +16,9 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_MRB_ID_H
-#define GRN_MRB_ID_H
+#pragma once
 
-#include "../ctx.h"
+#include "../grn_ctx.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -31,4 +30,3 @@ void grn_mrb_id_init(grn_ctx *ctx);
 }
 #endif
 
-#endif /* GRN_MRB_ID_H */
