@@ -17,7 +17,6 @@
 */
 
 #include "grn.h"
-#include "grn_db.h"
 #include "grn_index_column.h"
 
 grn_bool
@@ -73,11 +72,7 @@ grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj)
   if (!obj) { return GRN_FALSE; }
 
   id = grn_obj_id(ctx, obj);
-  if (id == GRN_ID_NIL) {
-    return GRN_FALSE;
-  } else {
-    return id < GRN_N_RESERVED_TYPES;
-  }
+  return grn_id_is_builtin(ctx, id);
 }
 
 grn_bool
@@ -148,6 +143,16 @@ grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj)
   }
 }
 
+grn_bool
+grn_obj_is_index_column(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_column(ctx, obj)) {
+    return GRN_FALSE;
+  }
+
+  return obj->header.type == GRN_COLUMN_INDEX;
+}
+
 grn_bool
 grn_obj_is_accessor(grn_ctx *ctx, grn_obj *obj)
 {
@@ -241,7 +246,7 @@ grn_obj_is_selector_proc(grn_ctx *ctx, grn_obj *obj)
   }
 
   proc = (grn_proc *)obj;
-  return proc->selector != NULL;
+  return proc->callbacks.function.selector != NULL;
 }
 
 grn_bool
@@ -296,6 +301,19 @@ grn_obj_is_scorer_proc(grn_ctx *ctx, grn_obj *obj)
   return proc->type == GRN_PROC_SCORER;
 }
 
+grn_bool
+grn_obj_is_window_function_proc(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_proc *proc;
+
+  if (!grn_obj_is_proc(ctx, obj)) {
+    return GRN_FALSE;
+  }
+
+  proc = (grn_proc *)obj;
+  return proc->type == GRN_PROC_WINDOW_FUNCTION;
+}
+
 static void
 grn_db_reindex(grn_ctx *ctx, grn_obj *db)
 {
@@ -507,3 +525,17 @@ grn_obj_type_to_string(uint8_t type)
     return "unknown";
   }
 }
+
+grn_bool
+grn_obj_name_is_column(grn_ctx *ctx, const char *name, int name_len)
+{
+  if (!name) {
+    return GRN_FALSE;
+  }
+
+  if (name_len < 0) {
+    name_len = strlen(name);
+  }
+
+  return memchr(name, GRN_DB_DELIMITER, name_len) != NULL;
+}
