@@ -28,7 +28,7 @@
 #endif
 
 #ifdef GRN_SUPPORT_REGEXP
-# include <oniguruma.h>
+# include <onigmo.h>
 #endif
 
 static const char *operator_names[] = {
@@ -361,6 +361,9 @@ grn_operator_exec_not_equal(grn_ctx *ctx, grn_obj *x, grn_obj *y)
 
 #define DO_COMPARE_SCALAR_SUB_NUMERIC(y,op) do {\
   switch ((y)->header.domain) {\
+  case GRN_DB_BOOL :\
+    r = (x_ op (uint8_t)(GRN_BOOL_VALUE(y) ? 1 : 0));\
+    break;\
   case GRN_DB_INT8 :\
     r = (x_ op GRN_INT8_VALUE(y));\
     break;\
@@ -455,6 +458,12 @@ grn_operator_exec_not_equal(grn_ctx *ctx, grn_obj *x, grn_obj *y)
 
 #define DO_COMPARE_SCALAR_BUILTIN(x,y,r,op) do {\
   switch (x->header.domain) {\
+  case GRN_DB_BOOL :\
+    {\
+      uint8_t x_ = GRN_BOOL_VALUE(x) ? 1 : 0;\
+      DO_COMPARE_SCALAR_SUB(op);\
+    }\
+    break;\
   case GRN_DB_INT8 :\
     {\
       int8_t x_ = GRN_INT8_VALUE(x);\
@@ -632,7 +641,11 @@ grn_operator_exec_not_equal(grn_ctx *ctx, grn_obj *x, grn_obj *y)
     }\
     GRN_OBJ_FIN(ctx, &element_buffer);\
   } else {\
-    DO_COMPARE_SCALAR(x, y, r, op);\
+    if (GRN_BULK_VSIZE(x) == 0 || GRN_BULK_VSIZE(y) == 0) {\
+      r = GRN_FALSE;\
+    } else {\
+      DO_COMPARE_SCALAR(x, y, r, op);\
+    }\
   }\
 } while (0)
 
@@ -731,7 +744,7 @@ static OnigRegex
 regexp_compile(grn_ctx *ctx,
                const char *pattern,
                unsigned int pattern_len,
-               OnigSyntaxType *syntax)
+               const OnigSyntaxType *syntax)
 {
   OnigRegex regex;
   OnigEncoding onig_encoding;
