@@ -948,13 +948,11 @@ grn_plugin_ensure_registered(grn_ctx *ctx, grn_obj *proc)
 grn_rc
 grn_plugin_get_names(grn_ctx *ctx, grn_obj *names)
 {
-  grn_obj *db;
-  grn_table_cursor *cursor;
-  grn_id id;
   grn_hash *processed_paths;
   const char *system_plugins_dir;
   const char *native_plugin_suffix;
   const char *ruby_plugin_suffix;
+  grn_bool is_close_opened_object_mode = GRN_FALSE;
 
   GRN_API_ENTER;
 
@@ -962,57 +960,63 @@ grn_plugin_get_names(grn_ctx *ctx, grn_obj *names)
     GRN_API_RETURN(ctx->rc);
   }
 
-  db = ctx->impl->db;
-  cursor = grn_table_cursor_open(ctx, db, NULL, 0, NULL, 0, 0, -1,
-                                 GRN_CURSOR_BY_ID);
-  if (!cursor) {
-    GRN_API_RETURN(ctx->rc);
+  if (grn_thread_get_limit() == 1) {
+    is_close_opened_object_mode = GRN_TRUE;
   }
 
   processed_paths = grn_hash_create(ctx, NULL, GRN_TABLE_MAX_KEY_SIZE, 0,
                                     GRN_OBJ_TABLE_HASH_KEY |
                                     GRN_OBJ_KEY_VAR_SIZE);
   if (!processed_paths) {
-    grn_table_cursor_close(ctx, cursor);
     GRN_API_RETURN(ctx->rc);
   }
 
   system_plugins_dir = grn_plugin_get_system_plugins_dir();
   native_plugin_suffix = grn_plugin_get_suffix();
   ruby_plugin_suffix = grn_plugin_get_ruby_suffix();
-  while ((id = grn_table_cursor_next(ctx, cursor)) != GRN_ID_NIL) {
+
+  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, grn_ctx_db(ctx), cursor, id,
+                             GRN_CURSOR_BY_ID | GRN_CURSOR_ASCENDING) {
+    void *name;
+    int name_size;
+    grn_bool is_opened = GRN_TRUE;
     grn_obj *object;
     const char *path;
     grn_id processed_path_id;
 
+    if (grn_id_is_builtin(ctx, id)) {
+      continue;
+    }
+
+    name_size = grn_table_cursor_get_key(ctx, cursor, &name);
+    if (grn_obj_name_is_column(ctx, name, name_size)) {
+      continue;
+    }
+
+    if (is_close_opened_object_mode) {
+      is_opened = grn_ctx_is_opened(ctx, id);
+    }
+
     object = grn_ctx_at(ctx, id);
     if (!object) {
       ERRCLR(ctx);
-      continue;
+      goto next_loop;
     }
 
     if (!grn_obj_is_proc(ctx, object)) {
-      grn_obj_unlink(ctx, object);
-      continue;
-    }
-
-    if (grn_obj_is_builtin(ctx, object)) {
-      grn_obj_unlink(ctx, object);
-      continue;
+      goto next_loop;
     }
 
     path = grn_obj_path(ctx, object);
     if (!path) {
-      grn_obj_unlink(ctx, object);
-      continue;
+      goto next_loop;
     }
 
     processed_path_id = grn_hash_get(ctx, processed_paths,
                                      path, strlen(path),
                                      NULL);
     if (processed_path_id != GRN_ID_NIL) {
-      grn_obj_unlink(ctx, object);
-      continue;
+      goto next_loop;
     }
 
     grn_hash_add(ctx, processed_paths,
@@ -1052,8 +1056,12 @@ grn_plugin_get_names(grn_ctx *ctx, grn_obj *names)
                              name, strlen(name),
                              0, GRN_DB_TEXT);
     }
-  }
-  grn_table_cursor_close(ctx, cursor);
+
+  next_loop :
+    if (object && is_close_opened_object_mode && !is_opened) {
+      grn_obj_close(ctx, object);
+    }
+  } GRN_TABLE_EACH_END(ctx, cursor);
 
   grn_hash_close(ctx, processed_paths);
 
