@@ -912,7 +912,7 @@ set_value(grn_ctx *ctx, grn_ja *ja, grn_id id, void *value, uint32_t value_len,
   return rc;
 }
 
-grn_rc
+static grn_rc
 grn_ja_put_raw(grn_ctx *ctx, grn_ja *ja, grn_id id,
                void *value, uint32_t value_len, int flags, uint64_t *cas)
 {
@@ -1200,49 +1200,220 @@ grn_ja_element_info(grn_ctx *ctx, grn_ja *ja, grn_id id,
   return GRN_SUCCESS;
 }
 
+#define COMPRESSED_VALUE_META_FLAG(meta) ((meta) & 0xf000000000000000)
+#define COMPRESSED_VALUE_META_FLAG_RAW             0x1000000000000000
+#define COMPRESSED_VALUE_META_UNCOMPRESSED_LEN(meta) \
+                                         ((meta) & 0x0fffffffffffffff)
+
+#define COMPRESS_THRESHOLD_BYTE 256
+#define COMPRESS_PACKED_VALUE_SIZE_MAX 257
+        /* COMPRESS_THRESHOLD_BYTE - 1 + sizeof(uint64_t) = 257 */
+
+#if defined(GRN_WITH_ZLIB) || defined(GRN_WITH_LZ4) || defined(GRN_WITH_ZSTD)
+#  define GRN_WITH_COMPRESSED
+#endif
+
+#ifdef GRN_WITH_COMPRESSED
+static void *
+grn_ja_ref_packed(grn_ctx *ctx,
+                  grn_io_win *iw,
+                  uint32_t *value_len,
+                  void *raw_value,
+                  uint32_t raw_value_len,
+                  void **compressed_value,
+                  uint32_t *compressed_value_len,
+                  uint32_t *uncompressed_value_len)
+{
+  uint64_t compressed_value_meta;
+
+  compressed_value_meta = *((uint64_t *)raw_value);
+  *compressed_value = (void *)(((uint64_t *)raw_value) + 1);
+  *compressed_value_len = raw_value_len - sizeof(uint64_t);
+
+  *uncompressed_value_len =
+    COMPRESSED_VALUE_META_UNCOMPRESSED_LEN(compressed_value_meta);
+  switch (COMPRESSED_VALUE_META_FLAG(compressed_value_meta)) {
+  case COMPRESSED_VALUE_META_FLAG_RAW :
+    iw->uncompressed_value = NULL;
+    *value_len = *uncompressed_value_len;
+    return *compressed_value;
+  default :
+    return NULL;
+  }
+}
+
+static grn_rc
+grn_ja_put_packed(grn_ctx *ctx,
+                  grn_ja *ja,
+                  grn_id id,
+                  void *value,
+                  uint32_t value_len,
+                  int flags,
+                  uint64_t *cas)
+{
+  char *packed_value[COMPRESS_PACKED_VALUE_SIZE_MAX];
+  uint32_t packed_value_len;
+  uint64_t packed_value_meta;
+
+  packed_value_len = value_len + sizeof(uint64_t);
+  packed_value_meta = value_len | COMPRESSED_VALUE_META_FLAG_RAW;
+  *((uint64_t *)packed_value) = packed_value_meta;
+  memcpy(((uint64_t *)packed_value) + 1,
+         value,
+         value_len);
+  return grn_ja_put_raw(ctx,
+                        ja,
+                        id,
+                        packed_value,
+                        packed_value_len,
+                        flags,
+                        cas);
+}
+
+static void
+grn_ja_compress_error(grn_ctx *ctx,
+                      grn_ja *ja,
+                      grn_id id,
+                      grn_rc rc,
+                      const char *message,
+                      const char *detail)
+{
+  char name[GRN_TABLE_MAX_KEY_SIZE];
+  int name_len;
+
+  if (ja->obj.id == GRN_ID_NIL) {
+    name[0] = '\0';
+    name_len = 0;
+  } else {
+    name_len = grn_obj_name(ctx, (grn_obj *)ja, name, GRN_TABLE_MAX_KEY_SIZE);
+  }
+  ERR(GRN_ZSTD_ERROR,
+      "[ja]%s: %s%.*s%s<%u>%s%s%s",
+      message,
+      name_len == 0 ? "" : "<",
+      name_len,
+      name,
+      name_len == 0 ? "" : ">: ",
+      id,
+      detail ? " :<" : "",
+      detail ? detail : "",
+      detail ? ">" : "");
+}
+#endif /* GRN_WITH_COMPRESSED */
+
 #ifdef GRN_WITH_ZLIB
 #include <zlib.h>
 
+static const char *
+grn_zrc_to_string(int zrc)
+{
+  switch (zrc) {
+  case Z_OK :
+    return "OK";
+  case Z_STREAM_END :
+    return "Stream is end";
+  case Z_NEED_DICT :
+    return "Need dictionary";
+  case Z_ERRNO :
+    return "See errno";
+  case Z_STREAM_ERROR :
+    return "Stream error";
+  case Z_DATA_ERROR :
+    return "Data error";
+  case Z_MEM_ERROR :
+    return "Memory error";
+  case Z_BUF_ERROR :
+    return "Buffer error";
+  case Z_VERSION_ERROR :
+    return "Version error";
+  default :
+    return "Unknown";
+  }
+}
+
 static void *
 grn_ja_ref_zlib(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_io_win *iw, uint32_t *value_len)
 {
   z_stream zstream;
+  void *raw_value;
+  uint32_t raw_value_len;
   void *zvalue;
   uint32_t zvalue_len;
-  if (!(zvalue = grn_ja_ref_raw(ctx, ja, id, iw, &zvalue_len))) {
+  void *unpacked_value;
+  uint32_t uncompressed_value_len;
+  int zrc;
+
+  if (!(raw_value = grn_ja_ref_raw(ctx, ja, id, iw, &raw_value_len))) {
     iw->uncompressed_value = NULL;
     *value_len = 0;
     return NULL;
   }
-  zstream.next_in = (Bytef *)(((uint64_t *)zvalue) + 1);
-  zstream.avail_in = zvalue_len - sizeof(uint64_t);
+
+  unpacked_value = grn_ja_ref_packed(ctx,
+                                     iw, value_len,
+                                     raw_value, raw_value_len,
+                                     &zvalue, &zvalue_len,
+                                     &uncompressed_value_len);
+  if (unpacked_value) {
+    return unpacked_value;
+  }
+
+  zstream.next_in = (Bytef *)zvalue;
+  zstream.avail_in = zvalue_len;
   zstream.zalloc = Z_NULL;
   zstream.zfree = Z_NULL;
-  if (inflateInit2(&zstream, 15 /* windowBits */) != Z_OK) {
+  zrc = inflateInit2(&zstream, 15 /* windowBits */);
+  if (zrc != Z_OK) {
     iw->uncompressed_value = NULL;
     *value_len = 0;
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZLIB_ERROR,
+                          "[zlib] failed to decompress: initialize",
+                          grn_zrc_to_string(zrc));
     return NULL;
   }
-  if (!(iw->uncompressed_value = GRN_MALLOC(*((uint64_t *)zvalue)))) {
+  if (!(iw->uncompressed_value = GRN_MALLOC(uncompressed_value_len))) {
     inflateEnd(&zstream);
     iw->uncompressed_value = NULL;
     *value_len = 0;
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZLIB_ERROR,
+                          "[zlib] failed to decompress: allocate buffer",
+                          NULL);
     return NULL;
   }
   zstream.next_out = (Bytef *)iw->uncompressed_value;
-  zstream.avail_out = *(uint64_t *)zvalue;
-  if (inflate(&zstream, Z_FINISH) != Z_STREAM_END) {
+  zstream.avail_out = uncompressed_value_len;
+  zrc = inflate(&zstream, Z_FINISH);
+  if (zrc != Z_STREAM_END) {
     inflateEnd(&zstream);
     GRN_FREE(iw->uncompressed_value);
     iw->uncompressed_value = NULL;
     *value_len = 0;
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZLIB_ERROR,
+                          "[zlib] failed to decompress: finish",
+                          grn_zrc_to_string(zrc));
     return NULL;
   }
   *value_len = zstream.total_out;
-  if (inflateEnd(&zstream) != Z_OK) {
+  zrc = inflateEnd(&zstream);
+  if (zrc != Z_OK) {
     GRN_FREE(iw->uncompressed_value);
     iw->uncompressed_value = NULL;
     *value_len = 0;
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZLIB_ERROR,
+                          "[zlib] failed to decompress: end",
+                          grn_zrc_to_string(zrc));
     return NULL;
   }
   return iw->uncompressed_value;
@@ -1255,53 +1426,132 @@ grn_ja_ref_zlib(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_io_win *iw, uint32_t *v
 static void *
 grn_ja_ref_lz4(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_io_win *iw, uint32_t *value_len)
 {
-  void *packed_value;
-  int packed_value_len;
+  void *raw_value;
+  uint32_t raw_value_len;
   void *lz4_value;
-  int lz4_value_len;
-  int original_value_len;
+  uint32_t lz4_value_len;
+  void *unpacked_value;
+  uint32_t uncompressed_value_len;
 
-  if (!(packed_value = grn_ja_ref_raw(ctx, ja, id, iw, &packed_value_len))) {
+  if (!(raw_value = grn_ja_ref_raw(ctx, ja, id, iw, &raw_value_len))) {
     iw->uncompressed_value = NULL;
     *value_len = 0;
     return NULL;
   }
-  original_value_len = *((uint64_t *)packed_value);
-  if (!(iw->uncompressed_value = GRN_MALLOC(original_value_len))) {
+
+  unpacked_value = grn_ja_ref_packed(ctx,
+                                     iw, value_len,
+                                     raw_value, raw_value_len,
+                                     &lz4_value, &lz4_value_len,
+                                     &uncompressed_value_len);
+  if (unpacked_value) {
+    return unpacked_value;
+  }
+
+  if (!(iw->uncompressed_value = GRN_MALLOC(uncompressed_value_len))) {
     iw->uncompressed_value = NULL;
     *value_len = 0;
     return NULL;
   }
-  lz4_value = (void *)((uint64_t *)packed_value + 1);
-  lz4_value_len = packed_value_len - sizeof(uint64_t);
   if (LZ4_decompress_safe((const char *)(lz4_value),
                           (char *)(iw->uncompressed_value),
                           lz4_value_len,
-                          original_value_len) < 0) {
+                          uncompressed_value_len) < 0) {
     GRN_FREE(iw->uncompressed_value);
     iw->uncompressed_value = NULL;
     *value_len = 0;
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_LZ4_ERROR,
+                          "[lz4] failed to decompress",
+                          NULL);
     return NULL;
   }
-  *value_len = original_value_len;
+  *value_len = uncompressed_value_len;
   return iw->uncompressed_value;
 }
 #endif /* GRN_WITH_LZ4 */
 
+#ifdef GRN_WITH_ZSTD
+#include <zstd.h>
+
+static void *
+grn_ja_ref_zstd(grn_ctx *ctx,
+                grn_ja *ja,
+                grn_id id,
+                grn_io_win *iw,
+                uint32_t *value_len)
+{
+  void *raw_value;
+  uint32_t raw_value_len;
+  void *zstd_value;
+  uint32_t zstd_value_len;
+  void *unpacked_value;
+  uint32_t uncompressed_value_len;
+  size_t written_len;
+
+  if (!(raw_value = grn_ja_ref_raw(ctx, ja, id, iw, &raw_value_len))) {
+    iw->uncompressed_value = NULL;
+    *value_len = 0;
+    return NULL;
+  }
+
+  unpacked_value = grn_ja_ref_packed(ctx,
+                                     iw, value_len,
+                                     raw_value, raw_value_len,
+                                     &zstd_value, &zstd_value_len,
+                                     &uncompressed_value_len);
+  if (unpacked_value) {
+    return unpacked_value;
+  }
+
+  if (!(iw->uncompressed_value = GRN_MALLOC(uncompressed_value_len))) {
+    iw->uncompressed_value = NULL;
+    *value_len = 0;
+    return NULL;
+  }
+
+  written_len = ZSTD_decompress((char *)(iw->uncompressed_value),
+                                uncompressed_value_len,
+                                zstd_value,
+                                zstd_value_len);
+  if (ZSTD_isError(written_len)) {
+    GRN_FREE(iw->uncompressed_value);
+    iw->uncompressed_value = NULL;
+    *value_len = 0;
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZSTD_ERROR,
+                          "[zstd] failed to decompress",
+                          ZSTD_getErrorName(written_len));
+    return NULL;
+  }
+  *value_len = uncompressed_value_len;
+  return iw->uncompressed_value;
+}
+#endif /* GRN_WITH_ZSTD */
+
 void *
 grn_ja_ref(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_io_win *iw, uint32_t *value_len)
 {
+  switch (ja->header->flags & GRN_OBJ_COMPRESS_MASK) {
 #ifdef GRN_WITH_ZLIB
-  if (ja->header->flags & GRN_OBJ_COMPRESS_ZLIB) {
+  case GRN_OBJ_COMPRESS_ZLIB :
     return grn_ja_ref_zlib(ctx, ja, id, iw, value_len);
-  }
 #endif /* GRN_WITH_ZLIB */
 #ifdef GRN_WITH_LZ4
-  if (ja->header->flags & GRN_OBJ_COMPRESS_LZ4) {
+  case GRN_OBJ_COMPRESS_LZ4 :
     return grn_ja_ref_lz4(ctx, ja, id, iw, value_len);
-  }
 #endif /* GRN_WITH_LZ4 */
-  return grn_ja_ref_raw(ctx, ja, id, iw, value_len);
+#ifdef GRN_WITH_ZSTD
+  case GRN_OBJ_COMPRESS_ZSTD :
+    return grn_ja_ref_zstd(ctx, ja, id, iw, value_len);
+#endif /* GRN_WITH_ZSTD */
+  default :
+    return grn_ja_ref_raw(ctx, ja, id, iw, value_len);
+  }
 }
 
 grn_obj *
@@ -1343,36 +1593,68 @@ grn_ja_put_zlib(grn_ctx *ctx, grn_ja *ja, grn_id id,
   z_stream zstream;
   void *zvalue;
   int zvalue_len;
+  int zrc;
 
   if (value_len == 0) {
     return grn_ja_put_raw(ctx, ja, id, value, value_len, flags, cas);
   }
 
+  if (value_len < COMPRESS_THRESHOLD_BYTE) {
+    return grn_ja_put_packed(ctx, ja, id, value, value_len, flags, cas);
+  }
+
   zstream.next_in = value;
   zstream.avail_in = value_len;
   zstream.zalloc = Z_NULL;
   zstream.zfree = Z_NULL;
-  if (deflateInit2(&zstream, Z_DEFAULT_COMPRESSION, Z_DEFLATED,
-                   15 /* windowBits */,
-                   8 /* memLevel */,
-                   Z_DEFAULT_STRATEGY) != Z_OK) {
-    ERR(GRN_ZLIB_ERROR, "deflateInit2 failed");
+  zrc = deflateInit2(&zstream, Z_DEFAULT_COMPRESSION, Z_DEFLATED,
+                     15 /* windowBits */,
+                     8 /* memLevel */,
+                     Z_DEFAULT_STRATEGY);
+  if (zrc != Z_OK) {
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZLIB_ERROR,
+                          "[zlib] failed to compress: initialize",
+                          grn_zrc_to_string(zrc));
     return ctx->rc;
   }
   zvalue_len = deflateBound(&zstream, value_len);
-  if (!(zvalue = GRN_MALLOC(zvalue_len + sizeof(uint64_t)))) { deflateEnd(&zstream); return GRN_NO_MEMORY_AVAILABLE; }
+  if (!(zvalue = GRN_MALLOC(zvalue_len + sizeof(uint64_t)))) {
+    deflateEnd(&zstream);
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZLIB_ERROR,
+                          "[zlib] failed to allocate compress buffer",
+                          NULL);
+    return ctx->rc;
+  }
   zstream.next_out = (Bytef *)(((uint64_t *)zvalue) + 1);
   zstream.avail_out = zvalue_len;
-  if (deflate(&zstream, Z_FINISH) != Z_STREAM_END) {
+  zrc = deflate(&zstream, Z_FINISH);
+  if (zrc != Z_STREAM_END) {
     deflateEnd(&zstream);
     GRN_FREE(zvalue);
-    ERR(GRN_ZLIB_ERROR, "deflate failed");
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZLIB_ERROR,
+                          "[zlib] failed to compress: finish",
+                          grn_zrc_to_string(zrc));
     return ctx->rc;
   }
   zvalue_len = zstream.total_out;
-  if (deflateEnd(&zstream) != Z_OK) {
+  zrc = deflateEnd(&zstream);
+  if (zrc != Z_OK) {
     GRN_FREE(zvalue);
-    ERR(GRN_ZLIB_ERROR, "deflateEnd failed");
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZLIB_ERROR,
+                          "[zlib] failed to compress: end",
+                          grn_zrc_to_string(zrc));
     return ctx->rc;
   }
   *(uint64_t *)zvalue = value_len;
@@ -1389,57 +1671,172 @@ grn_ja_put_lz4(grn_ctx *ctx, grn_ja *ja, grn_id id,
 {
   grn_rc rc;
   void *packed_value;
-  int packed_value_len;
+  int packed_value_len_max;
+  int packed_value_len_real;
   char *lz4_value;
-  int lz4_value_len;
+  int lz4_value_len_max;
+  int lz4_value_len_real;
 
   if (value_len == 0) {
     return grn_ja_put_raw(ctx, ja, id, value, value_len, flags, cas);
   }
 
+  if (value_len < COMPRESS_THRESHOLD_BYTE) {
+    return grn_ja_put_packed(ctx, ja, id, value, value_len, flags, cas);
+  }
+
   if (value_len > (uint32_t)LZ4_MAX_INPUT_SIZE) {
-    ERR(GRN_INVALID_ARGUMENT,
-        "[ja][lz4] too large value size: <%u>: max: <%d>",
-        value_len, LZ4_MAX_INPUT_SIZE);
+    uint64_t packed_value_meta;
+
+    packed_value_len_real = value_len + sizeof(uint64_t);
+    packed_value = GRN_MALLOC(packed_value_len_real);
+    if (!packed_value) {
+      grn_ja_compress_error(ctx,
+                            ja,
+                            id,
+                            GRN_LZ4_ERROR,
+                            "[lz4] failed to allocate packed buffer",
+                            NULL);
+      return ctx->rc;
+    }
+    packed_value_meta = value_len | COMPRESSED_VALUE_META_FLAG_RAW;
+    *((uint64_t *)packed_value) = packed_value_meta;
+    memcpy(((uint64_t *)packed_value) + 1,
+           value,
+           value_len);
+    rc = grn_ja_put_raw(ctx,
+                        ja,
+                        id,
+                        packed_value,
+                        packed_value_len_real,
+                        flags,
+                        cas);
+    GRN_FREE(packed_value);
+    return rc;
+  }
+
+  lz4_value_len_max = LZ4_compressBound(value_len);
+  packed_value_len_max = lz4_value_len_max + sizeof(uint64_t);
+  if (!(packed_value = GRN_MALLOC(packed_value_len_max))) {
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_LZ4_ERROR,
+                          "[lz4] failed to allocate compress buffer",
+                          NULL);
     return ctx->rc;
   }
-
-  lz4_value_len = LZ4_compressBound(value_len);
-
-  if (!(packed_value = GRN_MALLOC(lz4_value_len + sizeof(uint64_t)))) {
-    return GRN_NO_MEMORY_AVAILABLE;
-  }
   lz4_value = (char *)((uint64_t *)packed_value + 1);
-  lz4_value_len = LZ4_compress((const char*)value, lz4_value, value_len);
-
-  if (lz4_value_len <= 0) {
+  lz4_value_len_real = LZ4_compress((const char*)value, lz4_value, value_len);
+  if (lz4_value_len_real <= 0) {
     GRN_FREE(packed_value);
-    ERR(GRN_LZ4_ERROR, "LZ4_compress");
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_LZ4_ERROR,
+                          "[lz4] failed to compress",
+                          NULL);
     return ctx->rc;
   }
   *(uint64_t *)packed_value = value_len;
-  packed_value_len = lz4_value_len + sizeof(uint64_t);
-  rc = grn_ja_put_raw(ctx, ja, id, packed_value, packed_value_len, flags, cas);
+  packed_value_len_real = lz4_value_len_real + sizeof(uint64_t);
+  rc = grn_ja_put_raw(ctx,
+                      ja,
+                      id,
+                      packed_value,
+                      packed_value_len_real,
+                      flags,
+                      cas);
   GRN_FREE(packed_value);
   return rc;
 }
 #endif /* GRN_WITH_LZ4 */
 
+#ifdef GRN_WITH_ZSTD
+inline static grn_rc
+grn_ja_put_zstd(grn_ctx *ctx,
+                grn_ja *ja,
+                grn_id id,
+                void *value,
+                uint32_t value_len,
+                int flags,
+                uint64_t *cas)
+{
+  grn_rc rc;
+  void *packed_value;
+  int packed_value_len_max;
+  int packed_value_len_real;
+  void *zstd_value;
+  int zstd_value_len_max;
+  int zstd_value_len_real;
+  int zstd_compression_level = 3;
+
+  if (value_len == 0) {
+    return grn_ja_put_raw(ctx, ja, id, value, value_len, flags, cas);
+  }
+
+  if (value_len < COMPRESS_THRESHOLD_BYTE) {
+    return grn_ja_put_packed(ctx, ja, id, value, value_len, flags, cas);
+  }
+
+  zstd_value_len_max = ZSTD_compressBound(value_len);
+  packed_value_len_max = zstd_value_len_max + sizeof(uint64_t);
+  if (!(packed_value = GRN_MALLOC(packed_value_len_max))) {
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZSTD_ERROR,
+                          "[zstd] failed to allocate compress buffer",
+                          NULL);
+    return ctx->rc;
+  }
+  zstd_value = ((uint64_t *)packed_value) + 1;
+  zstd_value_len_real = ZSTD_compress(zstd_value, zstd_value_len_max,
+                                      value, value_len,
+                                      zstd_compression_level);
+  if (ZSTD_isError(zstd_value_len_real)) {
+    grn_ja_compress_error(ctx,
+                          ja,
+                          id,
+                          GRN_ZSTD_ERROR,
+                          "[zstd] failed to compress",
+                          ZSTD_getErrorName(zstd_value_len_real));
+    return ctx->rc;
+  }
+  *(uint64_t *)packed_value = value_len;
+  packed_value_len_real = zstd_value_len_real + sizeof(uint64_t);
+  rc = grn_ja_put_raw(ctx,
+                      ja,
+                      id,
+                      packed_value,
+                      packed_value_len_real,
+                      flags,
+                      cas);
+  GRN_FREE(packed_value);
+  return rc;
+}
+#endif /* GRN_WITH_ZSTD */
+
 grn_rc
 grn_ja_put(grn_ctx *ctx, grn_ja *ja, grn_id id, void *value, uint32_t value_len,
            int flags, uint64_t *cas)
 {
+  switch (ja->header->flags & GRN_OBJ_COMPRESS_MASK) {
 #ifdef GRN_WITH_ZLIB
-  if (ja->header->flags & GRN_OBJ_COMPRESS_ZLIB) {
+  case GRN_OBJ_COMPRESS_ZLIB :
     return grn_ja_put_zlib(ctx, ja, id, value, value_len, flags, cas);
-  }
 #endif /* GRN_WITH_ZLIB */
 #ifdef GRN_WITH_LZ4
-  if (ja->header->flags & GRN_OBJ_COMPRESS_LZ4) {
+  case GRN_OBJ_COMPRESS_LZ4 :
     return grn_ja_put_lz4(ctx, ja, id, value, value_len, flags, cas);
-  }
 #endif /* GRN_WITH_LZ4 */
-  return grn_ja_put_raw(ctx, ja, id, value, value_len, flags, cas);
+#ifdef GRN_WITH_ZSTD
+  case GRN_OBJ_COMPRESS_ZSTD :
+    return grn_ja_put_zstd(ctx, ja, id, value, value_len, flags, cas);
+#endif /* GRN_WITH_ZSTD */
+  default :
+    return grn_ja_put_raw(ctx, ja, id, value, value_len, flags, cas);
+  }
 }
 
 static grn_rc
@@ -1685,7 +2082,7 @@ grn_ja_reader_close(grn_ctx *ctx, grn_ja_reader *reader)
   return rc;
 }
 
-#if defined(GRN_WITH_ZLIB) || defined(GRN_WITH_LZ4)
+#ifdef GRN_WITH_COMPRESSED
 /* grn_ja_reader_seek_compressed() prepares to access a compressed value. */
 static grn_rc
 grn_ja_reader_seek_compressed(grn_ctx *ctx, grn_ja_reader *reader, grn_id id)
@@ -1732,7 +2129,7 @@ grn_ja_reader_seek_compressed(grn_ctx *ctx, grn_ja_reader *reader, grn_id id)
   reader->value_size = (uint32_t)*(uint64_t *)seg_addr;
   return GRN_SUCCESS;
 }
-#endif /* defined(GRN_WITH_ZLIB) || defined(GRN_WITH_LZ4) */
+#endif /* GRN_WITH_COMPRESSED */
 
 /* grn_ja_reader_seek_raw() prepares to access a value. */
 static grn_rc
@@ -1783,17 +2180,22 @@ grn_ja_reader_seek_raw(grn_ctx *ctx, grn_ja_reader *reader, grn_id id)
 grn_rc
 grn_ja_reader_seek(grn_ctx *ctx, grn_ja_reader *reader, grn_id id)
 {
+  switch (reader->ja->header->flags & GRN_OBJ_COMPRESS_MASK) {
 #ifdef GRN_WITH_ZLIB
-  if (reader->ja->header->flags & GRN_OBJ_COMPRESS_ZLIB) {
+  case GRN_OBJ_COMPRESS_ZLIB :
     return grn_ja_reader_seek_compressed(ctx, reader, id);
-  }
 #endif /* GRN_WITH_ZLIB */
 #ifdef GRN_WITH_LZ4
-  if (reader->ja->header->flags & GRN_OBJ_COMPRESS_LZ4) {
+  case GRN_OBJ_COMPRESS_LZ4 :
     return grn_ja_reader_seek_compressed(ctx, reader, id);
-  }
 #endif /* GRN_WITH_LZ4 */
-  return grn_ja_reader_seek_raw(ctx, reader, id);
+#ifdef GRN_WITH_ZSTD
+  case GRN_OBJ_COMPRESS_ZSTD :
+    return grn_ja_reader_seek_compressed(ctx, reader, id);
+#endif /* GRN_WITH_ZSTD */
+  default :
+    return grn_ja_reader_seek_raw(ctx, reader, id);
+  }
 }
 
 grn_rc
@@ -1976,6 +2378,67 @@ grn_ja_reader_read_lz4(grn_ctx *ctx, grn_ja_reader *reader, void *buf)
 }
 #endif /* GRN_WITH_LZ4 */
 
+#ifdef GRN_WITH_ZSTD
+/* grn_ja_reader_read_zstd() reads a value compressed with Zstandard. */
+static grn_rc
+grn_ja_reader_read_zstd(grn_ctx *ctx, grn_ja_reader *reader, void *buf)
+{
+  int src_size, dest_size;
+  grn_ja_einfo *einfo = (grn_ja_einfo *)reader->einfo;
+  if (EHUGE_P(einfo)) {
+    grn_io *io = reader->ja->io;
+    void *seg_addr;
+    char *packed_ptr;
+    uint32_t size, seg_id;
+    if (reader->packed_size > reader->packed_buf_size) {
+      void *new_buf = GRN_REALLOC(reader->packed_buf, reader->packed_size);
+      if (!new_buf) {
+        return GRN_NO_MEMORY_AVAILABLE;
+      }
+      reader->packed_buf = new_buf;
+      reader->packed_buf_size = reader->packed_size;
+    }
+    packed_ptr = (char *)reader->packed_buf;
+    grn_memcpy(packed_ptr, (char *)reader->body_seg_addr + sizeof(uint64_t),
+               io->header->segment_size - sizeof(uint64_t));
+    packed_ptr += io->header->segment_size - sizeof(uint64_t);
+    size = reader->packed_size - (io->header->segment_size - sizeof(uint64_t));
+    seg_id = reader->body_seg_id + 1;
+    while (size > io->header->segment_size) {
+      GRN_IO_SEG_REF(io, seg_id, seg_addr);
+      if (!seg_addr) {
+        return GRN_UNKNOWN_ERROR;
+      }
+      grn_memcpy(packed_ptr, seg_addr, io->header->segment_size);
+      GRN_IO_SEG_UNREF(io, seg_id);
+      seg_id++;
+      size -= io->header->segment_size;
+      packed_ptr += io->header->segment_size;
+    }
+    GRN_IO_SEG_REF(io, seg_id, seg_addr);
+    if (!seg_addr) {
+      return GRN_UNKNOWN_ERROR;
+    }
+    grn_memcpy(packed_ptr, seg_addr, size);
+    GRN_IO_SEG_UNREF(io, seg_id);
+    seg_id++;
+    src_size = (int)(reader->packed_size - sizeof(uint64_t));
+    dest_size = ZSTD_decompress(reader->packed_buf, reader->value_size,
+                                buf, src_size);
+  } else {
+    char *packed_addr = (char *)reader->body_seg_addr;
+    packed_addr += reader->body_seg_offset + sizeof(uint64_t);
+    src_size = (int)(reader->packed_size - sizeof(uint64_t));
+    dest_size = ZSTD_decompress(packed_addr, reader->value_size,
+                                buf, src_size);
+  }
+  if ((uint32_t)dest_size != reader->value_size) {
+    return GRN_ZSTD_ERROR;
+  }
+  return GRN_SUCCESS;
+}
+#endif /* GRN_WITH_ZSTD */
+
 /* grn_ja_reader_read_raw() reads a value. */
 static grn_rc
 grn_ja_reader_read_raw(grn_ctx *ctx, grn_ja_reader *reader, void *buf)
@@ -2023,17 +2486,22 @@ grn_ja_reader_read_raw(grn_ctx *ctx, grn_ja_reader *reader, void *buf)
 grn_rc
 grn_ja_reader_read(grn_ctx *ctx, grn_ja_reader *reader, void *buf)
 {
+  switch (reader->ja->header->flags & GRN_OBJ_COMPRESS_MASK) {
 #ifdef GRN_WITH_ZLIB
-  if (reader->ja->header->flags & GRN_OBJ_COMPRESS_ZLIB) {
+  case GRN_OBJ_COMPRESS_ZLIB :
     return grn_ja_reader_read_zlib(ctx, reader, buf);
-  }
 #endif /* GRN_WITH_ZLIB */
 #ifdef GRN_WITH_LZ4
-  if (reader->ja->header->flags & GRN_OBJ_COMPRESS_LZ4) {
+  case GRN_OBJ_COMPRESS_LZ4 :
     return grn_ja_reader_read_lz4(ctx, reader, buf);
-  }
 #endif /* GRN_WITH_LZ4 */
-  return grn_ja_reader_read_raw(ctx, reader, buf);
+#ifdef GRN_WITH_ZSTD
+  case GRN_OBJ_COMPRESS_ZSTD :
+    return grn_ja_reader_read_zstd(ctx, reader, buf);
+#endif /* GRN_WITH_ZSTD */
+  default :
+    return grn_ja_reader_read_raw(ctx, reader, buf);
+  }
 }
 
 #ifdef GRN_WITH_ZLIB
@@ -2058,6 +2526,17 @@ grn_ja_reader_pread_lz4(grn_ctx *ctx, grn_ja_reader *reader,
 }
 #endif /* GRN_WITH_LZ4 */
 
+#ifdef GRN_WITH_ZSTD
+/* grn_ja_reader_pread_zstd() reads a part of a value compressed with ZSTD. */
+static grn_rc
+grn_ja_reader_pread_zstd(grn_ctx *ctx, grn_ja_reader *reader,
+                         size_t offset, size_t size, void *buf)
+{
+  /* TODO: To be supported? */
+  return GRN_FUNCTION_NOT_IMPLEMENTED;
+}
+#endif /* GRN_WITH_ZSTD */
+
 /* grn_ja_reader_pread_raw() reads a part of a value. */
 static grn_rc
 grn_ja_reader_pread_raw(grn_ctx *ctx, grn_ja_reader *reader,
@@ -2125,17 +2604,22 @@ grn_rc
 grn_ja_reader_pread(grn_ctx *ctx, grn_ja_reader *reader,
                     size_t offset, size_t size, void *buf)
 {
+  switch (reader->ja->header->flags & GRN_OBJ_COMPRESS_MASK) {
 #ifdef GRN_WITH_ZLIB
-  if (reader->ja->header->flags & GRN_OBJ_COMPRESS_ZLIB) {
+  case GRN_OBJ_COMPRESS_ZLIB :
     return grn_ja_reader_pread_zlib(ctx, reader, offset, size, buf);
-  }
 #endif /* GRN_WITH_ZLIB */
 #ifdef GRN_WITH_LZ4
-  if (reader->ja->header->flags & GRN_OBJ_COMPRESS_LZ4) {
+  case GRN_OBJ_COMPRESS_LZ4 :
     return grn_ja_reader_pread_lz4(ctx, reader, offset, size, buf);
-  }
 #endif /* GRN_WITH_LZ4 */
-  return grn_ja_reader_pread_raw(ctx, reader, offset, size, buf);
+#ifdef GRN_WITH_ZSTD
+  case GRN_OBJ_COMPRESS_ZSTD :
+    return grn_ja_reader_pread_zstd(ctx, reader, offset, size, buf);
+#endif /* GRN_WITH_ZSTD */
+  default :
+    return grn_ja_reader_pread_raw(ctx, reader, offset, size, buf);
+  }
 }
 
 /**** vgram ****/
